/*
  factorize.c

  Copyright (C) 2015 Taiji Yamada <taiji@aihara.co.jp>
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>
#include "prime_util.h"

void stdout_factorize(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  huge_uint f = 2, c;

  printf("%llu\t", n);
  if (n == 1) printf("%llu^%llu\t", 1ULL, 0ULL);
  for (; f <= n; f = isprime_bit_table_nextprime(u, isprime_bit_table, f)) {
    c = 0;
    while (n % f == 0) {
      ++c;
      n /= f;
    }
    if (c > 0) printf("%llu^%llu\t", f, c);
    if (!(f < u)) abort();
  }
  printf("\n");
}

int usage(FILE *fp, int argc, char *argv[])
{
  (void)argc;
  fprintf(fp, "Usage:\n\t%s [number ...]\n", basename(argv[0]));
  return fp == stderr ? 1 : 0;
}
int main(int argc, char *argv[])
{
  int a, fd;
  huge_uint n, u = 1000000000, n_return;
  isprime_bit_table_element_t *isprime_bit_table;

  if (argc < 2) return usage(stderr, argc, argv);
  isprime_bit_table = isprime_bit_table_mmap(u, &n_return, &fd);
  if (!isprime_bit_table) abort();
  primes_bit_continued_eratosthenes_sieve(n_return, isprime_bit_table, u);
  for (a=1; a<argc; ++a) {
    n = atoll(argv[a]);
    stdout_factorize(max(n_return, u), isprime_bit_table, n);
  }
  return 0;
}
