/*
  naturals.c

  Copyright (C) 2015 Taiji Yamada <taiji@aihara.co.jp>
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>
#include "prime_util.h"

enum naturals_type {
  natural,
  odd,
  even,
  singly_even,
  doubly_even,
  composite,
  prime,
  sophie_germain_prime,
  interprime,
  twin_prime,
  cousin_primes,
  sexy_primes,
  square_free,
  cube_free,
  perfect_square,
  perfect_power,
  powerful,
  //
  highlycomposite,
  highlyabundant,
  superabundant,
  abundant,
  primitive_abundant,
  primitive_abundant_have_no_abundant_proper_divisor,
  non_prime_unusual,
  //
  pseudoperfect, semiperfect,
  primitive_pseudoperfect, primitive_semiperfect,
  perfect,
  multiperfect,
  deficient,
  superperfect,
  hemiperfect,
  refactorable,
  unitary_perfect,
  primary_pseudoperfect,
  weird,
  practical,
  ruth_aaron_pair,
  primorial_ruth_aaron_pair,
  unusual,
  //
  arithmetic,
  harmonic_divisor,
  round_number,
  lucas_carmichael,
  semiprime,
  sphenic,
  amicable,
  betrothed,
  augumented_amicable,
  friendly,
  //
  mersenne,
  mersenne_prime,
  wilson_quotient,
  wilson_prime,
  //
  fortunate,
  hilbert,
  pronic,
  power_of_two,
  cullen,
  cullen_prime,
  woodall,
  woodall_prime,
  regular,
  factorial_prime,
  euclid,
  euclid_prime,
  kummer,
  kummer_prime,
  primorial_prime,
  triangular,
  triangular_pyramidal, tetrahedral,
  pentatope,
  pentagonal,
  square_triangular,
  squared_triangular,
  lazy_caterer,
  jacobsthal,
  jacobsthal_lucas,
  kynea,
  prime_kynea,
  thabit,
  prime_thabit,
  fermat,
  catalan,
  derangement,
  nontotient,
  noncototient,
  perfect_totient,
  highly_totient,
  highly_cototient,
  odd_semifactrial,
  even_semifactrial,
  //
  fibonacci,
  leonardo,
  lucas,
  pell,
  perrin,
  padovan,
  motzkin,
  schreder_hipparchus,
  sylvester,
  telephone,
  alternating_factorial,
  hyperfactorial,
  exponential_factorial,
};
#define S(x) #x
const char *naturals_labels[] = {
  S(natural),
  S(odd),
  S(even),
  S(singly_even),
  S(doubly_even),
  S(composite),
  S(prime),
  S(sophie_germain_prime),
  S(interprime),
  S(twin_prime),
  S(cousin_primes),
  S(sexy_primes),
  S(square_free),
  S(cube_free),
  S(perfect_square),
  S(perfect_power),
  S(powerful),
  //
  S(highlycomposite),
  S(highlyabundant),
  S(superabundant),
  S(abundant),
  S(primitive_abundant),
  S(primitive_abundant_have_no_abundant_proper_divisor),
  S(non_prime_unusual),
  //
  S(pseudoperfect), S(semiperfect),
  S(primitive_pseudoperfect), S(primitive_semiperfect),
  S(perfect),
  S(multiperfect),
  S(deficient),
  S(superperfect),
  S(hemiperfect),
  S(refactorable),
  S(unitary_perfect),
  S(primary_pseudoperfect),
  S(weird),
  S(practical),
  S(ruth_aaron_pair),
  S(primorial_ruth_aaron_pair),
  S(unusual),
  //
  S(arithmetic),
  S(harmonic_divisor),
  S(round_number),
  S(lucas_carmichael),
  S(semiprime),
  S(sphenic),
  S(amicable),
  S(betrothed),
  S(augumented_amicable),
  S(friendly),
  //
  S(mersenne),
  S(mersenne_prime),
  S(wilson_quotient),
  S(wilson_prime),
  //
  S(fortunate),
  S(hilbert),
  S(pronic),
  S(power_of_two),
  S(cullen),
  S(cullen_prime),
  S(woodall),
  S(woodall_prime),
  S(regular),
  S(factorial_prime),
  S(euclid),
  S(euclid_prime),
  S(kummer),
  S(kummer_prime),
  S(primorial_prime),
  S(triangular),
  S(triangular_pyramidal), S(tetrahedral),
  S(pentatope),
  S(pentagonal),
  S(square_triangular),
  S(squared_triangular),
  S(lazy_caterer),
  S(jacobsthal),
  S(jacobsthal_lucas),
  S(kynea),
  S(prime_kynea),
  S(thabit),
  S(prime_thabit),
  S(fermat),
  S(catalan),
  S(derangement),
  S(nontotient),
  S(noncototient),
  S(perfect_totient),
  S(highly_totient),
  S(highly_cototient),
  S(odd_semifactrial),
  S(even_semifactrial),
  //
  S(fibonacci),
  S(leonardo),
  S(lucas),
  S(pell),
  S(perrin),
  S(padovan),
  S(motzkin),
  S(schreder_hipparchus),
  S(sylvester),
  S(telephone),
  S(alternating_factorial),
  S(hyperfactorial),
  S(exponential_factorial),
};
#undef S

int usage(FILE *fp, int argc, char *argv[])
{
  (void)argc;
  fprintf(fp, "Usage:\n\t%s [-type name] [-l lower] [-u upper]\n", basename(argv[0]));
  fprintf(fp, "\ntype names:\n");
  for (size_t i=0; i<sizeof(naturals_labels)/sizeof(*naturals_labels); ++i)
    fprintf(fp, "\t%s\n", naturals_labels[i]);
  return fp == stderr ? 1 : 0;
}
int main(int argc, char *argv[])
{
  enum naturals_type type = natural;
  int a, fd;
  huge_uint n, l = 1, u = 100, n_return;
  isprime_bit_table_element_t *isprime_bit_table;

  for (a=1; a<argc; ++a)
    if (strcmp(argv[a], "-h") == 0)
      return usage(stdout, argc, argv);
    else if (strcmp(argv[a], "-type") == 0 && a+1 < argc) {
      for (size_t i=0; i<sizeof(naturals_labels)/sizeof(*naturals_labels); ++i)
	if (strcmp(argv[a+1], naturals_labels[i]) == 0) {
	  ++a;
	  type = (enum naturals_type)i;
	  break;
	}
      switch (type) {
      case pronic:
      case triangular:
      case triangular_pyramidal: case tetrahedral:
      case pentatope:
      case pentagonal:
      case square_triangular:
      case squared_triangular:
      case jacobsthal:
      case derangement:
      case fibonacci:
      case pell:
      case perrin:
	l = 0;
	break;
      default:
	l = 1;
	break;
      }
    }
    else if (strcmp(argv[a], "-l") == 0 && a+1 < argc)
      l = atoll(argv[++a]);
    else if (strcmp(argv[a], "-u") == 0 && a+1 < argc)
      u = atoll(argv[++a]);
    else
      break;

  isprime_bit_table = isprime_bit_table_mmap(u, &n_return, &fd);
  if (!isprime_bit_table) abort();
  primes_bit_continued_eratosthenes_sieve(n_return, isprime_bit_table, u);

  n = l;
  switch (type) {
  case natural:
    for (; n < u; ++n)
      printf("%llu\n", n);
    break;
  case odd:
    if (n % 2 == 0) ++n;
    for (; n < u; n+=2)
      printf("%llu\n", n);
    break;
  case even:
    if (n % 2 == 1) ++n;
    for (; n < u; n+=2)
      printf("%llu\n", n);
    break;
  case singly_even:
    if (n % 2 == 1) ++n;
    if (n % 4 == 0) n += 2;
    for (; n < u; n+=4)
      printf("%llu\n", n);
    break;
  case doubly_even:
    n += 4 - n % 4;
    for (; n < u; n+=4)
      printf("%llu\n", n);
    break;
  case composite:
    n = isprime_bit_table_firstcomposite(u, isprime_bit_table, n);
    for (; n < u; n = isprime_bit_table_nextcomposite(u, isprime_bit_table, n))
      printf("%llu\n", n);
    break;
  case prime:
    n = isprime_bit_table_firstprime(u, isprime_bit_table, n);
    for (; n < u; n = isprime_bit_table_nextprime(u, isprime_bit_table, n))
      printf("%llu\n", n);
    break;
  case sophie_germain_prime:
    {
      n = isprime_bit_table_firstprime(u, isprime_bit_table, n);
      for (; n < u; n = isprime_bit_table_nextprime(u, isprime_bit_table, n)) {
	huge_uint m = n*2 + 1;
	if (!(m < max(n_return, u))) abort();
	if (isprime_bit(m)) printf("%llu\t%llu\n", n, m);
      }
    }
    break;
  case interprime:
    {
      huge_uint m;

      if (n < 3) n = 3;
      n = isprime_bit_table_firstprime(u, isprime_bit_table, n);
      m = isprime_bit_table_nextprime(u, isprime_bit_table, n);
      for (; m < u; n = m, m = isprime_bit_table_nextprime(u, isprime_bit_table, n))
	printf("%llu\n", (n + m)/2);
    }
    break;
  case twin_prime:
  case cousin_primes:
  case sexy_primes:
    {
      huge_uint d, c, i;
      switch (type) {
      case twin_prime:			d = 2; c = 2; break;
      case cousin_primes:		d = 4; c = 2; break;
      case sexy_primes: default:	d = 6; c = 2; break;
      }
      n = isprime_bit_table_firstprime(u, isprime_bit_table, n);
      for (; n < u; n = isprime_bit_table_nextprime(u, isprime_bit_table, n)) {
	for (i=1; i<c; ++i) {
	  huge_uint m = n + d*i;
	  if (!(m < max(n_return, u))) abort();
	  if (!isprime_bit(m)) break;
	}
	if (i==c) {
	  for (i=0; i<c; ++i)
	    printf("%llu\t", n + d*i);
	  printf("\n");
	}
      }
    }
    break;
  case square_free:
    for (; n<u; ++n)
      if (is_square_free(n))
	printf("%llu\n", n);
    break;
  case cube_free:
    for (; n<u; ++n)
      if (is_cube_free(n))
	printf("%llu\n", n);
    break;
  case perfect_square:
    for (; n<u; ++n)
      if (is_perfect_square(u, isprime_bit_table, n))
	printf("%llu\n", n);
    break;
  case perfect_power:
    for (; n<u; ++n)
      if (is_perfect_power(u, isprime_bit_table, n))
	printf("%llu\n", n);
    break;
  case powerful:
    for (; n<u; ++n)
      if (is_powerful(u, isprime_bit_table, n))
	printf("%llu\n", n);
    break;
  case highlycomposite:
    {
      huge_uint m = 0, c;

      for (n = 1; n < 3 && n < u; ++n)
	if ((c=n_divisors(n)) > m) {
	  if (!(n < l)) printf("%llu\n", n);
	  m = c;
	}
      for (; n < u; n = isprime_bit_table_nextcomposite(u, isprime_bit_table, n))
	if ((c=n_divisors(n)) > m) {
	  if (!(n < l)) printf("%llu\n", n);
	  m = c;
	}
    }
    break;
  case highlyabundant:
    {
      huge_uint m = 0, s;

      for (n = 1; n < 3 && n < u; ++n)
	if ((s=sum_divisors(n)) > m) {
	  if (!(n < l)) printf("%llu\n", n);
	  m = s;
	}
      for (; n < u; n = isprime_bit_table_nextcomposite(u, isprime_bit_table, n))
	if ((s=sum_divisors(n)) > m) {
	  if (!(n < l)) printf("%llu\n", n);
	  m = s;
	}
    }
    break;
  case superabundant:
    {
      huge_uint m = 0, s, o = 1;

      for (n = 1; n < 3 && n < u; ++n)
	if ((s=sum_divisors(n))*o > m*n) {
	  if (!(n < l)) printf("%llu\n", n);
	  m = s;
	  o = n;
	}
      for (; n < u; n = isprime_bit_table_nextcomposite(u, isprime_bit_table, n))
	if ((s=sum_divisors(n))*o > m*n) {
	  if (!(n < l)) printf("%llu\n", n);
	  m = s;
	  o = n;
	}
    }
    break;
  case abundant:
    for (n = isprime_bit_table_firstcomposite(u, isprime_bit_table, n); n < u; n = isprime_bit_table_nextcomposite(u, isprime_bit_table, n))
      if (is_abundant(n))
	printf("%llu\n", n);
    break;
  case primitive_abundant:
    for (n = isprime_bit_table_firstcomposite(u, isprime_bit_table, n); n < u; n = isprime_bit_table_nextcomposite(u, isprime_bit_table, n))
      if (is_primitive_abundant(n))
	printf("%llu\n", n);
    break;
  case primitive_abundant_have_no_abundant_proper_divisor:
    for (n = isprime_bit_table_firstcomposite(u, isprime_bit_table, n); n < u; n = isprime_bit_table_nextcomposite(u, isprime_bit_table, n))
      if (is_primitive_abundant_have_no_abundant_proper_divisor(n))
	printf("%llu\n", n);
    break;
  case non_prime_unusual:
    for (n = isprime_bit_table_firstcomposite(u, isprime_bit_table, n); n < u; n = isprime_bit_table_nextcomposite(u, isprime_bit_table, n))
      if (is_non_prime_unusual(u, isprime_bit_table, n))
	printf("%llu\n", n);
    break;
  case pseudoperfect:
  case semiperfect:
    for (n = isprime_bit_table_firstcomposite(u, isprime_bit_table, n); n < u; n = isprime_bit_table_nextcomposite(u, isprime_bit_table, n))
      if (is_pseudoperfect(n))
	printf("%llu\n", n);
    break;
  case primitive_pseudoperfect:
  case primitive_semiperfect:
    for (n = isprime_bit_table_firstcomposite(u, isprime_bit_table, n); n < u; n = isprime_bit_table_nextcomposite(u, isprime_bit_table, n))
      if (is_primitive_pseudoperfect(u, isprime_bit_table, n))
	printf("%llu\n", n);
    break;
  case perfect:
    for (; n < u; ++n)
      if (is_perfect(n))
	printf("%llu\n", n);
    break;
  case multiperfect:
    for (; n < u; ++n)
      if (is_multiperfect(n))
	printf("%llu\n", n);
    break;
  case deficient:
    for (; n < u; ++n)
      if (is_deficient(n))
	printf("%llu\n", n);
    break;
  case superperfect:
    for (; n < u; ++n)
      if (is_superperfect(n))
	printf("%llu\n", n);
    break;
  case hemiperfect:
    for (; n < u; ++n) {
      huge_uint k;
      if (is_hemiperfect(n, &k))
	printf("%llu\t%llu\n", n, k);
    }
    break;
  case refactorable:
    for (; n < u; ++n)
      if (is_refactorable(n))
	printf("%llu\n", n);
    break;
  case unitary_perfect:
    for (; n < u; ++n)
      if (is_unitary_perfect(n))
	printf("%llu\n", n);
    break;
  case primary_pseudoperfect:
    for (; n < u; ++n)
      if (is_primary_pseudoperfect(max(n_return, u), isprime_bit_table, n))
	printf("%llu\n", n);
    break;
  case weird:
    for (; n < u; ++n)
      if (is_weird(n))
	printf("%llu\n", n);
    break;
  case practical:
    for (; n < u; ++n)
      if (is_practical(n))
	printf("%llu\n", n);
    break;
  case ruth_aaron_pair:
    for (; n < u; ++n)
      if (is_ruth_aaron_pair(max(n_return, u), isprime_bit_table, n))
	printf("%llu\t%llu\n", n, n+1);
    break;
  case primorial_ruth_aaron_pair:
    for (; n < u; ++n)
      if (is_primorial_ruth_aaron_pair(max(n_return, u), isprime_bit_table, n))
	printf("%llu\t%llu\n", n, n+1);
    break;
  case unusual:
    for (; n < u; ++n)
      if (is_unusual(u, isprime_bit_table, n))
	printf("%llu\n", n);
    break;
  case arithmetic:
    for (; n < u; ++n) {
      huge_uint s, c;
      if (is_arithmetic(n, &s, &c))
	printf("%llu\t%llu\n", n, s/c);
    }
    break;
  case harmonic_divisor:
    for (; n < u; ++n)
      if (is_harmonic_divisor(n))
	printf("%llu\n", n);
    break;
  case round_number:
    for (; n < u; ++n)
      if (is_round_number(u, isprime_bit_table, n))
	printf("%llu\n", n);
    break;
  case lucas_carmichael:
    for (; n < u; ++n)
      if (is_lucas_carmichael(u, isprime_bit_table, n))
	printf("%llu\n", n);
    break;
  case semiprime:
    {
      huge_uint i, j;

      for (; n < u; ++n)
	for (i = 2; i < u; i = isprime_bit_table_nextprime(u, isprime_bit_table, i))
	  for (j = 2; j < u/i; j = isprime_bit_table_nextprime(u, isprime_bit_table, j))
	    if (n == i*j) {
	      printf("%llu\t%llu\t%llu\n", n, i, j);
	      i = u;
	      break;
	    }
    }
    break;
  case sphenic:
    {
      huge_uint i, j, k;

      for (; n < u; ++n) {
	for (i = 2; i < u; i = isprime_bit_table_nextprime(u, isprime_bit_table, i))
	  for (j = isprime_bit_table_nextprime(u, isprime_bit_table, i); j < u/i; j = isprime_bit_table_nextprime(u, isprime_bit_table, j))
	    for (k = isprime_bit_table_nextprime(u, isprime_bit_table, j); k < u/i/j; k = isprime_bit_table_nextprime(u, isprime_bit_table, k))
	      if (n == i*j*k) {
		printf("%llu\t%llu\t%llu\t%llu\n", n, i, j, k);
		i = u;
		j = u/i;
		break;
	      }
      }
    }
    break;
  case amicable:
    for (; n < u; ++n) {
      huge_uint s = sum_divisors(n) - n, m;
      for (m = n + 1; m < u; ++m)
	if (s == m && sum_divisors(m) - m == n)
	  printf("%llu\t%llu\n", n, m);
    }
    break;
  case betrothed:
    for (; n < u; ++n) {
      huge_uint s = sum_divisors(n) - n - 1, m;
      for (m = n + 1; m < u; ++m)
	if (s == m && sum_divisors(m) - m - 1 == n)
	  printf("%llu\t%llu\n", n, m);
    }
    break;
  case augumented_amicable:
    for (; n < u; ++n) {
      huge_uint s = sum_divisors(n) - n + 1, m;
      for (m = n + 1; m < u; ++m)
	if (s == m && sum_divisors(m) - m + 1 == n)
	  printf("%llu\t%llu\n", n, m);
    }
    break;
  case friendly:
    for (; n < u; ++n) {
      huge_uint s1 = sum_divisors(n), c1 = greatest_common_divisor(s1, n), d1 = n/c1, m;
      s1 /= c1;
      for (m = n + 1; m < u; ++m) {
	huge_uint s2 = sum_divisors(m), c2 = greatest_common_divisor(s2, m), d2 = m/c2;
	s2 /= c2;
	if (s1 == s2 && d1 == d2)
	  printf("%llu\t%llu\n", n, m);
      }
    }
    break;
  case mersenne:
    {
      for (huge_uint i=2; ; i = isprime_bit_table_nextprime(max(n_return, u), isprime_bit_table, i)) {
	n = function_mersenne(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case mersenne_prime:
    {
      for (huge_uint i=2; ; i = isprime_bit_table_nextprime(max(n_return, u), isprime_bit_table, i)) {
	n = function_mersenne(i);
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case wilson_quotient:
    {
      for (huge_uint i=2; ; i = isprime_bit_table_nextprime(max(n_return, u), isprime_bit_table, i)) {
	n = function_wilson_quotient(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case wilson_prime:
    {
      for (huge_uint i=2; ; i = isprime_bit_table_nextprime(max(n_return, u), isprime_bit_table, i)) {
	n = function_wilson_quotient(i);
	if (!(n < u)) break;
	if (is_wilson_prime(n, i))
	  if (!(n < l)) printf("%llu\t%llu\n", i, n);
      }
    }
    break;
  case fortunate:
    {
      huge_uint m, i, p;

      for (i = 1; ; ++i) {
	p = primorial_number(max(n_return, u), isprime_bit_table, i);
	for (m = 2; ; ++m) {
	  n = p + m;
	  if (!(n < u)) break;
	  if (isprime_bit(n)) {
	    if (!(n < l)) printf("%llu\t%llu\n", m, n);
	    break;
	  }
	}
	if (!(n < u)) break;
      }
    }
    break;
  case hilbert:
    {
      for (huge_uint i=0; ; ++i) {
	n = function_hilbert(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case pronic:
    {
      for (huge_uint i=0; ; ++i) {
	n = function_pronic(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case power_of_two:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_power_of_two(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case cullen:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_cullen(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\t%lu\n", n, i);
      }
    }
    break;
  case cullen_prime:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_cullen(i);
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\t%lu\n", n, i);
      }
    }
    break;
  case woodall:
    {
      for (unsigned long i=1; ; ++i) {
	n = function_woodall(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\t%lu\n", n, i);
      }
    }
    break;
  case woodall_prime:
    {
      for (unsigned long i=1; ; ++i) {
	n = function_woodall(i);
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\t%lu\n", n, i);
      }
    }
    break;
  case regular:
    {
#if !0
      for (; n < u; ++n) {
	huge_uint n_f, *f = prime_factors(u, isprime_bit_table, n, &n_f);
	size_t i;
	for (i=0; i<n_f; ++i)
	  if (!(f[i] == 2 || f[i] == 3 || f[i] == 5))
	    break;
	if (i == n_f)
	  printf("%llu\n", n);
      }
#else // ! in order
      huge_uint m[3];

      for (unsigned long i=0; ; ++i) {
	m[0] = power(5, i/9);
	m[1] = power(3, (i%9)/3);
	m[2] = power(2, (i%3));
	n = m[2]*m[1]*m[0];
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
#endif
    }
    break;
  case factorial_prime:
    {
      for (unsigned long i=0; ; ++i) {
	huge_uint f = factorial(i);
	n = f - 1;
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\n", n);
	n = f + 1;
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case euclid:
    {
      for (huge_uint i=1; ; ++i) {
	n = function_euclid(max(n_return, u), isprime_bit_table, i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case euclid_prime:
    {
      for (huge_uint i=1; ; ++i) {
	n = function_euclid(max(n_return, u), isprime_bit_table, i);
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case kummer:
    {
      for (huge_uint i=1; ; ++i) {
	n = function_kummer(max(n_return, u), isprime_bit_table, i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case kummer_prime:
    {
      for (huge_uint i=1; ; ++i) {
	n = function_kummer(max(n_return, u), isprime_bit_table, i);
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case primorial_prime:
    {
      for (huge_uint i=1; ; ++i) {
	n = primorial_number(max(n_return, u), isprime_bit_table, i) - 1;
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\t%llu\n", n, i);
	n += 2;
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\t%llu\n", n, i);
      }
    }
    break;
  case triangular:
    {
      for (huge_uint i=0; ; ++i) {
	n = function_triangular(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case tetrahedral:
    {
      for (huge_uint i=0; ; ++i) {
	n = function_tetrahedral(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case pentatope:
    {
      for (huge_uint i=0; ; ++i) {
	n = function_pentatope(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case pentagonal:
    {
      for (huge_uint i=0; ; ++i) {
	n = function_pentagonal(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case square_triangular:
    {
      for (huge_uint i=0; ; ++i) {
	n = function_triangular(i);
	if (!(n < u)) break;
	if (is_square_triangular(n))
	  if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case squared_triangular:
    {
      for (huge_uint i=0; ; ++i) {
	n = function_squared_triangular(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case lazy_caterer:
    {
      for (huge_uint i=0; ; ++i) {
	n = function_lazy_caterer(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case jacobsthal:
    {
      for (size_t i=0; ; ++i) {
	n = function_jacobsthal(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case jacobsthal_lucas:
    {
      for (size_t i=0; ; ++i) {
	n = function_jacobsthal_lucas(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case kynea:
    {
      for (size_t i=1; ; ++i) {
	n = function_kynea(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case prime_kynea:
    {
      for (size_t i=1; ; ++i) {
	n = function_kynea(i);
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case thabit:
    {
      for (size_t i=0; ; ++i) {
	n = function_thabit(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case prime_thabit:
    {
      for (size_t i=0; ; ++i) {
	n = function_thabit(i);
	if (!(n < u)) break;
	if (isprime_bit(n))
	  if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case fermat: // !
    {
      for (unsigned long i=0; ; ++i) {
	n = function_fermat(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case catalan:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_catalan(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case derangement:
    {
      for (unsigned long i=0; ; ++i) {
	n = subfactorial(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case nontotient:
    {
      huge_uint i, j;

      for (i=2; ; i+=2) {
	n = i*i;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == totient_function(u, isprime_bit_table, j))
	    break;
	if (j == n)
	  if (!(n < l)) printf("%llu\n", i);
      }
    }
    break;
  case noncototient:
    {
      huge_uint i, j;

      for (i=4; ; i+=2) {
	n = i*i;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == cototient_function(u, isprime_bit_table, j))
	    break;
	if (j == n)
	  if (!(n < l)) printf("%llu\n", i);
      }
    }
    break;
  case perfect_totient:
    {
      huge_uint t, s;

      for (; n < u; ++n) {
	t = n;
	s = 0;
	while ((t = totient_function(u, isprime_bit_table, t)) > 1)
	  s += t;
	if (n != 1) ++s;
	if (s == n)
	  if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case highly_totient:
    {
      huge_uint i, j, d = 0, c;

      for (i=1; ; i++) {
	c = 0;
	n = i*i*2;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == totient_function(u, isprime_bit_table, j))
	    ++c;
	if (d < c) {
	  if (!(n < l)) printf("%llu\t%llu\n", i, c);
	  d = c;
	}
      }
    }
    break;
  case highly_cototient:
    {
      huge_uint i, j, d = 0, c;

      for (i=1; ; ++i) {
	c = 0;
	n = i*i*2;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == cototient_function(u, isprime_bit_table, j))
	    ++c;
	if (d < c) {
	  if (!(n < l)) printf("%llu\t%llu\n", i, c);
	  d = c;
	}
      }
    }
    break;
  case odd_semifactrial:
    {
      huge_uint i;

      for (i=1; ; i+=2) {
	n = semifactorial(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case even_semifactrial:
    {
      huge_uint i;

      for (i=0; ; i+=2) {
	n = semifactorial(i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
      }
    }
    break;
  case fibonacci:
    {
      huge_uint F[] = { 0, 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) printf("%llu\n", F[t]);
      for (; ; ) {
	n = F[0] + F[1];
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
    }
    break;
  case leonardo:
    {
      huge_uint L[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(L)/sizeof(L[0]); ++t)
	if (!(L[t] < l)) printf("%llu\n", L[t]);
      for (; ; ) {
	n = L[0] + L[1] + 1;
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(L)/sizeof(L[0]); ++t) L[t] = L[t+1];
	L[t] = n;
      }
    }
    break;
  case lucas:
    {
      huge_uint L[] = { 2, 1, };
      size_t t;
      for (t=0; t<sizeof(L)/sizeof(L[0]); ++t)
	if (!(L[t] < l)) printf("%llu\n", L[t]);
      for (; ; ) {
	n = L[0] + L[1];
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(L)/sizeof(L[0]); ++t) L[t] = L[t+1];
	L[t] = n;
      }
    }
    break;
  case pell:
    {
      huge_uint P[] = { 0, 1, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) printf("%llu\n", P[t]);
      for (; ; ) {
	n = P[1]*2 + P[0];
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
    }
    break;
  case perrin:
    {
      huge_uint P[] = { 3, 0, 2, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) printf("%llu\n", P[t]);
      for (; ; ) {
	n = P[1] + P[0];
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
    }
    break;
  case padovan:
    {
      huge_uint P[] = { 1, 1, 1, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) printf("%llu\n", P[t]);
      for (; ; ) {
	n = P[0] + P[1];
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
    }
    break;
  case motzkin:
    {
      huge_uint P[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) printf("%llu\n", P[t]);
      for (huge_uint i=2; ; ++i) {
	n = (P[0]*3*(i-1) + P[1]*(2*i+1))/(2+i);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
    }
    break;
  case schreder_hipparchus:
    {
      huge_uint S[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(S)/sizeof(S[0]); ++t)
	if (!(S[t] < l)) printf("%llu\n", S[t]);
      for (huge_uint i=3; ; ++i) {
	n = ((6*i - 9)*S[1] - (i - 3)*S[0])/i;
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(S)/sizeof(S[0]); ++t) S[t] = S[t+1];
	S[t] = n;
      }
    }
    break;
  case sylvester:
    {
      huge_uint S[] = { 2, };
      size_t t;
      for (t=0; t<sizeof(S)/sizeof(S[0]); ++t)
	if (!(S[t] < l)) printf("%llu\n", S[t]);
      for (huge_uint i=1; ; ++i) {
	n = S[0]*(S[0] - 1) + 1;
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(S)/sizeof(S[0]); ++t) S[t] = S[t+1];
	S[t] = n;
      }
    }
    break;
  case telephone:
    {
      huge_uint T[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(T)/sizeof(T[0]); ++t)
	if (!(T[t] < l)) printf("%llu\n", T[t]);
      for (huge_uint i=2; ; ++i) {
	n = T[1] + (i - 1)*T[0];
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(T)/sizeof(T[0]); ++t) T[t] = T[t+1];
	T[t] = n;
      }
    }
    break;
  case alternating_factorial:
    {
      huge_uint F[] = { 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) printf("%llu\n", F[t]);
      for (unsigned long i=2; ; F[0] = n, ++i) {
	n = factorial(i) - F[0];
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
    }
    break;
  case hyperfactorial:
    {
      huge_uint F[] = { 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) printf("%llu\n", F[t]);
      for (unsigned long i=2; ; ++i) {
	n = power(i, i)*F[0];
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
    }
    break;
  case exponential_factorial: // !
    {
      huge_uint F[] = { 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) printf("%llu\n", F[t]);
      for (unsigned long i=1; ; ++i) {
	n = power(i, F[0]);
	if (!(n < u)) break;
	if (!(n < l)) printf("%llu\n", n);
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
    }
    break;
  default:
    break;
  }

  return 0;
}
